pub struct BinaryView {
pub handle: *mut BNBinaryView,
}Expand description
Represents the “whole view” of the binary and its analysis.
Analysis information:
Annotation information:
Data representation and binary information:
Cleaning up
BinaryView has a cyclic relationship with the associated FileMetadata, each holds a strong
reference to one another, so to properly clean up/free the BinaryView, you must manually close the
file using FileMetadata::close, this is not fixable in the general case, until FileMetadata
has only a weak reference to the BinaryView.
Fields
handle: *mut BNBinaryViewImplementations
Sourceimpl BinaryView
impl BinaryView
pub unsafe fn from_raw(handle: *mut BNBinaryView) -> Self
Sourcepub fn from_metadata(meta: &FileMetadata) -> Result<Ref<Self>>
pub fn from_metadata(meta: &FileMetadata) -> Result<Ref<Self>>
Construct the raw binary view from the given metadata.
Before calling this, make sure you have a valid file path set for the FileMetadata. It is
required that the FileMetadata::file_path exist in the local filesystem.
Sourcepub fn from_path(
meta: &FileMetadata,
file_path: impl AsRef<Path>,
) -> Result<Ref<Self>>
pub fn from_path( meta: &FileMetadata, file_path: impl AsRef<Path>, ) -> Result<Ref<Self>>
Construct the raw binary view from the given file_path and metadata.
This will implicitly set the metadata file path and then construct the view. If the metadata
already has the desired file path, use BinaryView::from_metadata instead.
Sourcepub unsafe fn from_accessor<A: Accessor>(
meta: &FileMetadata,
accessor: &mut FileAccessor<A>,
) -> Result<Ref<Self>>
pub unsafe fn from_accessor<A: Accessor>( meta: &FileMetadata, accessor: &mut FileAccessor<A>, ) -> Result<Ref<Self>>
Construct the raw binary view from the given accessor and metadata.
It is the responsibility of the caller to keep the accessor alive for the lifetime of the view; because of this, we mark the function as unsafe.
Sourcepub fn from_data(meta: &FileMetadata, data: &[u8]) -> Ref<Self>
pub fn from_data(meta: &FileMetadata, data: &[u8]) -> Ref<Self>
Construct the raw binary view from the given data and metadata.
The data will be copied into the view, so the caller does not need to keep the data alive.
Sourcepub fn save_to_path(&self, file_path: impl AsRef<Path>) -> bool
pub fn save_to_path(&self, file_path: impl AsRef<Path>) -> bool
Save the original binary file to the provided file_path along with any modifications.
WARNING: Currently there is a possibility to deadlock if the analysis has queued up a main thread action
that tries to take the FileMetadata lock of the current view, and is executed while we
are executing in this function.
To avoid the above issue use crate::main_thread::execute_on_main_thread_and_wait to verify there
are no queued up main thread actions.
Sourcepub fn save_to_accessor<A: Accessor>(&self, file: &mut FileAccessor<A>) -> bool
pub fn save_to_accessor<A: Accessor>(&self, file: &mut FileAccessor<A>) -> bool
Save the original binary file to the provided FileAccessor along with any modifications.
WARNING: Currently there is a possibility to deadlock if the analysis has queued up a main thread action
that tries to take the FileMetadata lock of the current view, and is executed while we
are executing in this function.
To avoid the above issue use crate::main_thread::execute_on_main_thread_and_wait to verify there
are no queued up main thread actions.